/**********************************************************************************
 * (c) 2015, Master Technology
 * Licensed under the MIT license or contact me for a Support or Commercial License
 *
 * I do contract work in most languages, so let me solve your problems!
 *
 * Any questions please feel free to email me or put a issue up on the github repo
 * Version 0.0.1                                      Nathan@master-technology.com
 *********************************************************************************
 * Please feel free to include this in any of your plugins so we can make more
 * standard post install routines
 *********************************************************************************/
"use strict";

/* global require, process, fs.existsSync, fs.readFileSync */

var exec = require('child_process').exec;
var fs = require('fs');
var path = require('path');
var os = require('os');
var crypto = require('crypto');


var installUtilities = function() {
    console.log("NativeScript Install Utilities (c)2015, Master Technology (http://www.master-technology.com).\n");
    var curDir = process.cwd();
    var idx = curDir.indexOf('node_modules');
    if (idx > 0) {
        curDir = curDir.substring(0,idx);
    }
    this._errorState = false;
    this._primaryDirectory = curDir;
    this._primarySettings = {platforms: {}};
    if (fs.existsSync(this._primaryDirectory+"package.json")) {
        try {
            this._primarySettings = JSON.parse(fs.readFileSync(this._primaryDirectory+"package.json"));
        } catch (e) {

        }
    }
    if (!this._primarySettings.nativescript) {
        this._errorState = true;
        console.log("Unable to detect or parse root project!")
    }
};

installUtilities.prototype.copyFile = function (src, dest, forceOverWrite) {
    if (!forceOverWrite && fs.existsSync(dest) || !fs.existsSync(src)) { return; }
    fs.writeFileSync(dest, fs.readFileSync(src));
};

installUtilities.prototype.updateModuleMap = function(module, src, dest) {
    if (fs.existsSync(dest)) {
        var buffer = fs.readFileSync(dest);
        // Check to see if this module already exists
        if (buffer.toString().indexOf(module) !== -1) { return; }

        var resultBuffer = Buffer.concat([buffer, new Buffer("\r\r"), fs.readFileSync(src)]);
        // Add our module to the end
        fs.writeFileSync(dest, resultBuffer);
    } else {
        copyFile(src, dest);
    }
};

installUtilities.prototype.getFileSha = function(filename, callback) {
    var shaSum = crypto.createHash('sha1');
    if (!fs.existsSync(filename)) {
        callback("File is missing: "+filename);
    }
    var readStream = fs.createReadStream(filename);
    readStream.on('data', function(d) {
        shaSum.update(d);
    });

    readStream.on('end', function() {
        var d = shaSum.digest('hex');
        callback(false, d);
    });
};

installUtilities.prototype.hasAndroid = function() {
    if (!fs.existsSync(this._primaryDirectory+"platforms/android/")) { return false; }
    return !!this.androidRuntimeVersion();
};

installUtilities.prototype.hasIOS = function() {
    if (!fs.existsSync(this._primaryDirectory+"platforms/ios/")) { return false; }
    return !!this.iosRuntimeVersion();
};

installUtilities.prototype.iosRuntimeVersion = function() {
    if (this._errorState) { return false; }
    for (var key in this._primarySettings.nativescript ) {
        if (this._primarySettings.nativescript.hasOwnProperty(key)) {
            if (key.indexOf("ios") >= 0) {
                return this._primarySettings.nativescript[key].version;
            }
        }
    }
    return false;
};

installUtilities.prototype.androidRuntimeVersion = function() {
    if (this._errorState) { return false; }
    for (var key in this._primarySettings.nativescript ) {
        if (this._primarySettings.nativescript.hasOwnProperty(key)) {
            if (key.indexOf("android") >= 0) {
                return this._primarySettings.nativescript[key].version;
            }
        }
    }
    return false;
};

installUtilities.prototype.commonVersion = function() {
    if (this._errorState) { return false; }
    var corePackage = {version: ""};
    try {
        corePackage = JSON.parse(fs.readFileSync(this._primaryDirectory + "app/tns_modules/package.json"));
    }
    catch (e) {}
    return corePackage.version;
};

installUtilities.prototype.IOSProjectName = function() {
    var files = fs.readdirSync(this._primaryDirectory+"platforms/ios/");
    var projectName = [];
    for (var i=0;i<files.length;i++) {
        var file = files[i];
        if (file.indexOf(".xcodeproj") > 0) {
            projectName.push(file);
        }
    }
    if (projectName.length === 0) { return ""; }
    else if (projectName.length === 1) { return projectName[0]; }
    else { return projectName; }
};

installUtilities.prototype.launchXcode = function(projectFile, callback) {
    if (!fs.existsSync(this._primaryDirectory+'platforms/ios/'+projectFile)) {
        callback("Missing ProjectFile: "+projectFile);
    }
    exec(["open", "-a", "Xcode", projectFile].join(' '), {cwd: this._primaryDirectory+'platforms/ios/'}, function (error) {
        if (callback) { callback(error); }
    });
};

installUtilities.prototype.installAndroidLibrary = function(library, callback) {
    this.installLibrary("android", library, callback);
};

installUtilities.prototype.installIOSLibrary = function(library, callback) {
    this.installLibrary("ios", library, callback);
};

installUtilities.prototype.installLibrary = function(platform, library, callback) {
    exec(["tns", "library", "add", platform, process.cwd()+library].join(' '),{cwd: this._primaryDirectory}, function (error) {
        if (callback) { callback(error); }
    });
};

installUtilities.prototype.installFolder = function(platform, folder, callback) {
    // Eliminate last "/" or "\\" if it is the last character
    if (folder[folder.length-1] === '/' || folder[folder.length-1] === '\\') {
        folder.length--;
    }
    var files =  fs.readdirSync(folder);
    var idx1 = folder.lastIndexOf('/');
    var idx2 = folder.lastIndexOf('\\');
    if (idx1 < idx2) {
        idx1 = idx2;
    }
    var destinationFolder;
    if (idx1 > -1) {
        destinationFolder = folder.substring(idx1+1);
    } else {
        destinationFolder = folder;
    }
    var fullDestinationFolder = this._primaryDirectory + "platforms/" + platform + "/" + destinationFolder;
    if (!fs.existsSync( fullDestinationFolder )) {
        fs.mkdirSync( fullDestinationFolder );
    }
    for (var i=0;i<files.length;i++) {
        if (platform === 'ios' && files[i] === "module.modulemap") {
            this.updateModuleMap(destinationFolder, folder + "/" + files[i], this._primaryDirectory + "platforms/ios/module.modulemap");
        } else {
            this.copyFile(folder + "/" + files[i], fullDestinationFolder + files[i], true);
        }
    }
};


module.exports = installUtilities;